RSpec.describe TalltyImportExport::Importable do
  before do
    class Test
      include TalltyImportExport::Importable

      class << self
        def model_headers
          [
            { key: 'name', name: '名称' },
            { key: 'code', name: '学号', convert: :handle_code },
            { key: 'sub_code', name: '副学号', convert: :skip },
            { key: 'meta1', name: 'meta字段1', json: :meta },
            { key: 'meta2', name: 'meta字段2', json: :meta },
            { key: 'sum', name: '累加值', convert: :handle_sum },
          ]
        end
      end

      class Import
        def import_record line_info, associations
          associations << line_info
        end

        def handle_code code, processing_line_info, raw_line_info
          "#{code}_#{raw_line_info['sub_code']}"
        end

        def handle_sum num, processing_line_info, raw_line_info
          context.last_line_info ? context.last_line_info['sum'] + num : num
        end
      end
    end
  end

  it 'import file directly' do
    # 名称	学号	  副学号	meta字段1	meta字段2
    # 1	  20070101	1	    meta1	    metaA
    # 2	  20070102	2	    meta2	    metaB
    # 3	  20070103	3	    meta3	    metaC
    # 4	  20070104	4	    meta4	    metaD
    # 5	  20070105	5	    meta5	    metaE
    @result = []
    Test.import_xlsx('./spec/test1.xlsx', @result, headers: [{ key: 'name', name: '名称' }])

    expect(@result).to eq([
      {"name"=>1, "code"=>"20070101_1", "sub_code"=>nil, "meta"=>{"meta1"=>"meta1", "meta2"=>"metaA" }, "sum" => 1 },
      {"name"=>2, "code"=>"20070102_2", "sub_code"=>nil, "meta"=>{"meta1"=>"meta2", "meta2"=>"metaB" }, "sum" => 3 },
      {"name"=>3, "code"=>"20070103_3", "sub_code"=>nil, "meta"=>{"meta1"=>"meta3", "meta2"=>"metaC" }, "sum" => 6 },
      {"name"=>4, "code"=>"20070104_4", "sub_code"=>nil, "meta"=>{"meta1"=>"meta4", "meta2"=>"metaD" }, "sum" => 10 },
      {"name"=>5, "code"=>"20070105_5", "sub_code"=>nil, "meta"=>{"meta1"=>"meta5", "meta2"=>"metaE" }, "sum" => 15 },
    ])
  end

  it 'import excel object' do
    # 名称	学号	  副学号	meta字段1	meta字段2
    # 1	  20070101	1	    meta1	    metaA
    # 2	  20070102	2	    meta2	    metaB
    # 3	  20070103	3	    meta3	    metaC
    # 4	  20070104	4	    meta4	    metaD
    # 5	  20070105	5	    meta5	    metaE
    @result = []

    @excel = TalltyImportExport::Excel.new
    @excel.load('./spec/test1.xlsx')

    Test.import_xlsx(@excel, @result)

    expect(@result).to eq([
      {"name"=>1, "code"=>"20070101_1", "sub_code"=>nil, "meta"=>{"meta1"=>"meta1", "meta2"=>"metaA" }, "sum" => 1 },
      {"name"=>2, "code"=>"20070102_2", "sub_code"=>nil, "meta"=>{"meta1"=>"meta2", "meta2"=>"metaB" }, "sum" => 3 },
      {"name"=>3, "code"=>"20070103_3", "sub_code"=>nil, "meta"=>{"meta1"=>"meta3", "meta2"=>"metaC" }, "sum" => 6 },
      {"name"=>4, "code"=>"20070104_4", "sub_code"=>nil, "meta"=>{"meta1"=>"meta4", "meta2"=>"metaD" }, "sum" => 10 },
      {"name"=>5, "code"=>"20070105_5", "sub_code"=>nil, "meta"=>{"meta1"=>"meta5", "meta2"=>"metaE" }, "sum" => 15 },
    ])
  end
end
